@@ -47,6 +47,22 @@ where
     alloc: A,
 }
 
+impl<T, A> Clone for KeyMap<T, A>
+where
+    T: Clone,
+    A: Allocator + Clone,
+{
+    fn clone(&self) -> Self {
+        let mut result = KeyMap::with_capacity(self.capacity, self.alloc.clone()).unwrap();
+
+        for (k, v) in self.iter() {
+            result.insert(k, v.clone()).unwrap();
+        }
+
+        result
+    }
+}
+
 impl<T, A: Allocator> std::fmt::Debug for KeyMap<T, A>
 where
     T: std::fmt::Debug,
@@ -60,6 +76,8 @@ where
 pub enum MapError {
     #[error("Failed to allocate memory {0}")]
     AllocError(crate::alloc::AllocError),
+    #[error("0 keys mean unintialized entries")]
+    InvalidHandle,
 }
 
 pub struct Entry<'a, T> {
@@ -223,7 +241,7 @@ where
         let new_cap = capacity + self.count;
         if new_cap > self.capacity {
             unsafe {
-                self.adjust_size((new_cap as f32 * (1.0 + MAX_LOAD)) as usize)?;
+                self.adjust_capacity((new_cap as f32 * (1.0 + MAX_LOAD)) as usize)?;
             }
         }
         Ok(())
@@ -321,6 +339,15 @@ where
         })
     }
 
+    pub fn iter_mut(&mut self) -> impl Iterator<Item = (Handle, &'_ mut T)> + '_ {
+        let keys = self.keys.as_ptr();
+        let values = self.values.as_ptr();
+        (0..self.capacity).filter_map(move |i| unsafe {
+            let k = *keys.add(i);
+            (k.0 != 0).then(|| (k, &mut *values.add(i)))
+        })
+    }
+
     unsafe fn alloc_storage(
         alloc: &A,
         capacity: usize,
@@ -345,13 +372,14 @@ where
         };
         // zero the keys
         let keys: NonNull<Handle> = transmute(keys);
-        for i in 0..capacity {
-            std::ptr::write(keys.as_ptr().add(i), Handle(0));
+        {
+            let keys = std::slice::from_raw_parts_mut(keys.as_ptr(), capacity);
+            keys.fill(Handle(0));
         }
         Ok((keys, transmute(values)))
     }
 
-    unsafe fn adjust_size(&mut self, capacity: usize) -> Result<(), MapError> {
+    unsafe fn adjust_capacity(&mut self, capacity: usize) -> Result<(), MapError> {
         let capacity = pad_pot(capacity).max(2); // allocate at least two items
         let (mut keys, mut values) = Self::alloc_storage(&self.alloc, capacity)?;
 
@@ -359,6 +387,7 @@ where
         swap(&mut self.values, &mut values);
 
         let old_cap = self.capacity;
+        let old_count = self.count;
         // insert the old values
         self.count = 0;
         self.capacity = capacity;
@@ -382,19 +411,26 @@ where
                 .expect("old T layout"),
         );
 
+        debug_assert_eq!(
+            old_count, self.count,
+            "Expected count to stay unchanged after capacity adjustments"
+        );
+
         Ok(())
     }
 
     #[inline]
     fn grow(&mut self) -> Result<(), MapError> {
-        let new_cap = self.capacity.max(2) * 3 / 2;
+        let new_cap = (self.capacity.max(2) * 3) / 2;
         debug_assert!(new_cap > self.capacity);
-        unsafe { self.adjust_size(new_cap) }
+        unsafe { self.adjust_capacity(new_cap) }
     }
 
     /// Returns mutable reference to the just inserted value
     pub fn insert(&mut self, key: Handle, value: T) -> Result<&mut T, MapError> {
-        debug_assert_ne!(key.0, 0, "0 keys mean unintialized entries");
+        if key.0 == 0 {
+            return Err(MapError::InvalidHandle);
+        }
         if (self.count + 1) as f32 > self.capacity as f32 * MAX_LOAD {
             self.grow()?;
         }
@@ -482,6 +518,22 @@ impl<T> IndexMut<u32> for KeyMap<T> {
     }
 }
 
+impl<T> Index<&str> for KeyMap<T> {
+    type Output = T;
+
+    fn index(&self, key: &str) -> &Self::Output {
+        let key = Handle::from_str(key).unwrap();
+        &self[key]
+    }
+}
+
+impl<T> IndexMut<&str> for KeyMap<T> {
+    fn index_mut(&mut self, key: &str) -> &mut Self::Output {
+        let key = Handle::from_str(key).unwrap();
+        &mut self[key]
+    }
+}
+
 impl<T> Index<&[u8]> for KeyMap<T> {
     type Output = T;
 
@@ -490,6 +542,7 @@ impl<T> Index<&[u8]> for KeyMap<T> {
         &self[key]
     }
 }
+
 impl<T> IndexMut<&[u8]> for KeyMap<T> {
     fn index_mut(&mut self, key: &[u8]) -> &mut Self::Output {
         let key = Handle::from_bytes(key);
