@@ -47,6 +47,31 @@ impl<'a, Aux> Vm<'a, Aux> {
         })
     }
 
+    /// Inserts the given value into the VM's runtime memory. Returns the inserted [[Value]]
+    pub fn insert_value(&mut self, value: OwnedValue) -> Result<Value, ExecutionErrorPayload> {
+        let res = match value {
+            OwnedValue::Nil => Value::Nil,
+            OwnedValue::String(s) => {
+                let res = self.init_string(s.as_str())?;
+                Value::String(res)
+            }
+            OwnedValue::Object(o) => {
+                let mut res = self.init_table()?;
+                for [k, v] in o.into_iter() {
+                    let k = self.insert_value(k)?;
+                    let v = self.insert_value(v)?;
+                    unsafe {
+                        res.as_mut().insert(k, v)?;
+                    }
+                }
+                Value::Object(res.as_ptr())
+            }
+            OwnedValue::Integer(x) => Value::Integer(x),
+            OwnedValue::Real(x) => Value::Real(x),
+        };
+        Ok(res)
+    }
+
     pub fn clear(&mut self) {
         self.runtime_data.clear();
     }
@@ -102,7 +127,7 @@ impl<'a, Aux> Vm<'a, Aux> {
         S: Into<Value>,
     {
         self.runtime_data
-            .stack
+            .value_stack
             .push(value.into())
             .map_err(|_| ExecutionErrorPayload::Stackoverflow)?;
         Ok(())
@@ -110,7 +135,7 @@ impl<'a, Aux> Vm<'a, Aux> {
 
     #[inline]
     pub fn stack_pop(&mut self) -> Value {
-        self.runtime_data.stack.pop()
+        self.runtime_data.value_stack.pop()
     }
 
     pub fn get_table(&self, value: Value) -> Result<&FieldTable, ExecutionErrorPayload> {
@@ -166,7 +191,7 @@ impl<'a, Aux> Vm<'a, Aux> {
     /// This mostly assumes that program is valid, produced by the compiler.
     /// As such running non-compiler emitted programs is very un-safe
     #[inline(never)]
-    pub fn run(&mut self, program: &CaoProgram) -> ExecutionResult<()> {
+    pub fn run(&mut self, program: &CaoCompiledProgram) -> ExecutionResult<()> {
         self.runtime_data
             .call_stack
             .push(CallFrame {
@@ -198,7 +223,7 @@ impl<'a, Aux> Vm<'a, Aux> {
             instr_ptr += 1;
             debug!(
                 "Executing: {:?} instr_ptr: {} Stack: {}",
-                instr, instr_ptr, self.runtime_data.stack
+                instr, instr_ptr, self.runtime_data.value_stack
             );
             match instr {
                 Instruction::InitTable => {
@@ -221,7 +246,7 @@ impl<'a, Aux> Vm<'a, Aux> {
                         .map_err(|err| payload_to_error(err, instr_ptr))?;
                 }
                 Instruction::SetProperty => {
-                    let [value, key, instance] = self.runtime_data.stack.pop_n::<3>();
+                    let [value, key, instance] = self.runtime_data.value_stack.pop_n::<3>();
                     let table = self
                         .get_table_mut(instance)
                         .map_err(|err| payload_to_error(err, instr_ptr))?;
@@ -276,7 +301,7 @@ impl<'a, Aux> Vm<'a, Aux> {
                     }
                 }
                 Instruction::GotoIfTrue => {
-                    let condition = self.runtime_data.stack.pop();
+                    let condition = self.runtime_data.value_stack.pop();
                     let pos: i32 =
                         unsafe { instr_execution::decode_value(&program.bytecode, &mut instr_ptr) };
                     debug_assert!(pos >= 0);
@@ -285,7 +310,7 @@ impl<'a, Aux> Vm<'a, Aux> {
                     }
                 }
                 Instruction::GotoIfFalse => {
-                    let condition = self.runtime_data.stack.pop();
+                    let condition = self.runtime_data.value_stack.pop();
                     let pos: i32 =
                         unsafe { instr_execution::decode_value(&program.bytecode, &mut instr_ptr) };
                     debug_assert!(pos >= 0);
@@ -310,7 +335,7 @@ impl<'a, Aux> Vm<'a, Aux> {
                     .stack_push(Value::Nil)
                     .map_err(|err| payload_to_error(err, instr_ptr))?,
                 Instruction::ClearStack => {
-                    self.runtime_data.stack.clear_until(
+                    self.runtime_data.value_stack.clear_until(
                         self.runtime_data
                             .call_stack
                             .last()
@@ -361,7 +386,7 @@ impl<'a, Aux> Vm<'a, Aux> {
                 Instruction::Pass => {}
                 Instruction::ScalarInt => {
                     self.runtime_data
-                        .stack
+                        .value_stack
                         .push(Value::Integer(unsafe {
                             instr_execution::decode_value(&program.bytecode, &mut instr_ptr)
                         }))
@@ -370,8 +395,8 @@ impl<'a, Aux> Vm<'a, Aux> {
                 }
                 Instruction::ScalarFloat => {
                     self.runtime_data
-                        .stack
-                        .push(Value::Floating(unsafe {
+                        .value_stack
+                        .push(Value::Real(unsafe {
                             instr_execution::decode_value(&program.bytecode, &mut instr_ptr)
                         }))
                         .map_err(|_| ExecutionErrorPayload::Stackoverflow)
@@ -441,7 +466,7 @@ impl<'a, Aux> Vm<'a, Aux> {
         let a = self.stack_pop();
 
         self.runtime_data
-            .stack
+            .value_stack
             .push(op(a, b))
             .map_err(|_| ExecutionErrorPayload::Stackoverflow)?;
         Ok(())
@@ -460,7 +485,7 @@ impl<'a, Aux> Vm<'a, Aux> {
                 Handle::from_str(s).unwrap()
             }
             Value::Integer(i) => Handle::from(i),
-            Value::Floating(_) | Value::Object(_) => return Err(ExecutionErrorPayload::Unhashable),
+            Value::Real(_) | Value::Object(_) => return Err(ExecutionErrorPayload::Unhashable),
         };
         Ok(handle)
     }
